package com.gemho.environment.service.impl;

import cn.hutool.core.util.NumberUtil;
import com.gemho.environment.model.Relay;
import com.gemho.environment.model.RelayDto;
import com.gemho.environment.service.RelayService;
import com.gemho.environment.utils.RedisUtils;
import com.gemho.environment.utils.ValidationUtil;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

/**
* @author 靖小侠
* @date 2020-05-07
*/
@Slf4j
@Service
//@CacheConfig(cacheNames = "relay")
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class RelayServiceImpl implements RelayService {

    private final RedisUtils redisUtils;

    private final RelayRepository relayRepository;

    private final RelayMapper relayMapper;

    private final UserRelayRepository userRelayRepository;

    private final SensorDeviceRepository sensorDeviceRepository;

    @Value("${smart.offlineCount}")
    private String offlineCount;

    public RelayServiceImpl(RedisUtils redisUtils, RelayRepository relayRepository, RelayMapper relayMapper, UserRelayRepository userRelayRepository, SensorDeviceRepository sensorDeviceRepository) {
        this.redisUtils = redisUtils;
        this.relayRepository = relayRepository;
        this.relayMapper = relayMapper;
        this.userRelayRepository = userRelayRepository;
        this.sensorDeviceRepository = sensorDeviceRepository;
    }

    @Override
    //@Cacheable(key = "#p0")
    public RelayDto findById(Long id) {
        Relay relay = relayRepository.findById(id).orElseGet(Relay::new);
        ValidationUtil.isNull(relay.getId(),"Relay","id",id);
        return relayMapper.toDto(relay);
    }

    @Override
    //@CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    public RelayDto create(Relay resources) {
//        resources.setId(IdUtil.simpleUUID());
        return relayMapper.toDto(relayRepository.save(resources));
    }

    @Override
    //@CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    public void update(Relay resources) {
        Relay relay = relayRepository.findById(resources.getId()).orElseGet(Relay::new);
        ValidationUtil.isNull( relay.getId(),"Relay","id",resources.getId());
        relay.copy(resources);
        relayRepository.save(relay);
    }

    @Override
    //@CacheEvict(allEntries = true)
    public void deleteAll(Long[] ids) {
        for (Long id : ids) {
            relayRepository.deleteById(id);
        }
    }

    @Override
    public void download(List<RelayDto> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (RelayDto relay : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("所属公司", relay.getCompany());
            map.put("IMEI码", relay.getImei());
            map.put("电信编码", relay.getTelecomcode());
            map.put("设备id", relay.getDeviceid());
            map.put("监测设备类型", relay.getDevicetype());
            map.put("设备名称", relay.getName());
            map.put("描述", relay.getNt());
            map.put("设备类型 0:nb 1:4g", relay.getType());
            map.put("安装位置", relay.getAddress());
            map.put("经度", relay.getJd());
            map.put("纬度", relay.getWd());
            map.put("信号强度", relay.getRssi());
            map.put("电池电量", relay.getBattery());
            map.put("是否有效", relay.getValid());
            map.put("ICCID或卡号", relay.getPhonecode());
            map.put("ip", relay.getIp());
            map.put("端口", relay.getPort());
            map.put("二维码", relay.getQrcode());
            map.put("创建时间", relay.getCreateTime());
            map.put("更新时间", relay.getUpdateTime());
            map.put("工作模式 0手动1自动", relay.getWorkmode());
            map.put("在线时间", relay.getOnlinetime());
            map.put("在线", relay.getOnline());
            map.put("0: 普通设备 1:控制器 2:无线LED屏幕", relay.getRelay());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

    @Override
    public RelayDto findByImei(String imei) {
        return relayMapper.toDto(relayRepository.findByImei(imei));
    }

    @Override
    public RelayDto findByDeviceid(String deviceid) {
        return relayMapper.toDto(relayRepository.findByDeviceid(deviceid));
    }



}
