package com.gemho.environment.service.impl;


import com.gemho.environment.model.SensorDto;
import com.gemho.environment.repository.SensorRepository;
import com.gemho.environment.service.SensorService;
import com.gemho.environment.service.mapper.SensorMapper;
import com.gemho.environment.utils.ValidationUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import com.gemho.environment.model.Sensor;


import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
* @author 靖小侠
* @date 2020-05-07
*/
@Service
//@CacheConfig(cacheNames = "sensor")
@Transactional(propagation = Propagation.SUPPORTS, readOnly = true, rollbackFor = Exception.class)
public class SensorServiceImpl implements SensorService {

    private final SensorRepository sensorRepository;

    private final SensorMapper sensorMapper;


    public SensorServiceImpl(SensorRepository sensorRepository, SensorMapper sensorMapper) {
        this.sensorRepository = sensorRepository;
        this.sensorMapper = sensorMapper;
    }

    @Override
    public SensorDto findByDeviceid(String deviceid) {
        return sensorMapper.toDto(sensorRepository.findByDeviceid(deviceid));
    }

    @Override
    //@CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    public void update(Sensor resources) {
        Sensor sensor = sensorRepository.findById(resources.getId()).orElseGet(Sensor::new);
        ValidationUtil.isNull( sensor.getId(),"Sensor","id",resources.getId());
        sensor.copy(resources);
        sensorRepository.save(sensor);
    }



}
